package wtalk.protocols.pop;

/*
 * Copyright (C) 2002 Matthew Wronka <wronkm@rpi.edu>
 * This file is released under the GNU General Public License version 2,
 * a version of which should have been provided along with this file.
 *
 * You might also find a copy at:
 * http://dmc.sf.net/files/gpl.txt
 * http://www.fsf.org/copyleft/gpl.html
 */

import wtalk.RMsgHandler;
import wtalk.IUserList;
import wtalk.protocols.WTalkProtocol;

import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Date;

import javax.mail.*;
import javax.mail.internet.*;
import java.io.IOException;

/**
 * Supports sending mail using SMTP
 * and retrieving mail using POP3
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 0
 */

public class POP3SMTP implements WTalkProtocol {
	private String mName;	// user, or user@host string
	private String mLogin;	// user in user@host
	private String mHost;	// host in user@host (if exists)
	private String mPasswd;	// password
	private String mServer;	// Mail server

	private POP3Checker popc = new POP3Checker();
	private Session mSession = null;

	// RMsgHandlers
	private RMsgHandler dmh = null;	// Default Message Handler
	private RMsgHandler sph = null; // Private Message Handler
	private RMsgHandler smh = null; // System Message Handler


	final static String VERSION = "20021201-0338";
	final static String NAME = "POP3SMTP plugin for WTalk";
	final static String AUTHOR = "Matthew Wronka <wronkm@rpi.edu>";

	final static String URL = "http://dmc.sourceforge.net/";

	static public String getBanner() {
		return NAME + " v" + VERSION + " by " + AUTHOR;
	}

	public POP3SMTP() {
		System.out.println(getBanner());
	}

	private void Reinstate() {
		popc.ReportTo(dmh);
		popc.ReportSystemTo(smh);
		popc.ReportPrivateTo(sph);
		popc.setLogin(mLogin);
		popc.setPasswd(mPasswd);
		popc.setServer(mHost);
	}

	//////////////////////////////////////////////////////////
	//
	// Callbacks
	//

	//
	// These are for writing messages ...
	//
	/**
	 * Sets default message handler.
	 * 
	 * @param Class which implements wtalk.RMsgHandler
	 * @see wtalk.RMsgHandler
	 */
	public void setReceivedMessageHandler(RMsgHandler r){	// Default
		dmh = r;
		popc.ReportTo(r);
	}

	/**
	 * Sets message handler for chat mesages. 
	 * 
	 * @param Class which implements wtalk.RMsgHandler
	 * @see wtalk.RMsgHandler
	 */
	public void setChatMessageHandler(RMsgHandler r){}	// Chats

	/**
	 * Sets message handler for private messages.
	 * 
	 * @param Class which implements wtalk.RMsgHandler
	 * @see wtalk.RMsgHandler
	 */
	public void setPrivateMessageHandler(RMsgHandler r){	// Private
		sph = r;
		popc.ReportPrivateTo(r);
	}

	/**
	 * Sets message handler for global brodcasts (walls).
	 * 
	 * @param Class which implements wtalk.RMsgHandler
	 * @see wtalk.RMsgHandler
	 */
	public void setWallMessageHandler(RMsgHandler r){}	// Walls

	/**
	 * Sets message handler for system messages.
	 * 
	 * @param Class which implements wtalk.RMsgHandler
	 * @see wtalk.RMsgHandler
	 */
	public void setSystemMessageHandler(RMsgHandler r){	// System..
		smh = r;
		popc.ReportSystemTo(r);
	}

	/**
	 * Sets handler for automated messages.
	 * 
	 * @param Class which implements wtalk.RMsgHandler
	 * @see wtalk.RMsgHandler
	 */
	public void setAutoMessageHandler(RMsgHandler r){}	// Auto..


	public void setClientMessageHandler(RMsgHandler r){}	// Auto..

	//
	// Sets call back for manipulating buddy-lists.
	//
	/**
	 * WTalk requires some file to handle lists of users,
	 * 	known as "Buddy Lists".  This class must
	 *	implement IUserList, though it need not actually
	 *	do anything with the the information provided.
	 *
	 * @param ul Class which manages User Lists.
	 * @see wtalk.IUserList
	 */
	public void setUserListHandler(IUserList ul){}
	//
	//////////////////////////////////////////////////////////


	//////////////////////////////////////////////////////////
	//
	// Uncooked functions
	//
	/**
	 * Sends an "unformatted" or "raw" message.
	 * <p>
	 * Not currently supported. 
	 *
	 * @param A string to send as a "raw" message.
	 */
	public void sendRaw(String msg) {}	// sends literal string
	//public void interpret(String cmd);	// Interpret a command
	//
	//////////////////////////////////////////////////////////


	//////////////////////////////////////////////////////////
	//
	// Authentication/Login
	//
	/**
	 * Connects to a preset server and port.
	 *
	 * @throws IOException Throws excetption if it cannot connect 
	 *	to the host; for example, invalid hostname or 
	 *	no service running.
	 */
	public void connect() throws IOException {
		//Authenticator auth = new ;
		Properties props = System.getProperties();
		props.put("mail.smtp.host", mServer);
		mSession = Session.getDefaultInstance(props, null);
		popc = new POP3Checker();
		Reinstate();
		popc.start();
	}

	/**
	 * Overloaded to set user name to 'login', and then connects 
	 *	to a preset server and port.
	 * 
	 * @throws IOException Throws excetption if it cannot connect 
	 *	to the host; for example, invalid hostname or 
	 *	no service running.
	 */
	public void connect(String login) throws IOException {
		setName(login); connect();
	}

	/**
	 * Overloaded to connects to a specified server and port--and 
	 *	remembers these values so that connect() will use them.
	 *
	 * @throws IOException Throws excetption if it cannot connect 
	 *	to the host; for example, invalid hostname or 
	 *	no service running.
	 */
	public void connect(String server, int port) throws IOException{
		setServer(server);	setServerPort(port);	connect();
	}

	/**
	 * Overloaded to set user name to 'login' and then connects 
	 *	to a specified server and port--and remembers these 
	 *	values so that connect() will use them.
	 *
	 * @throws IOException Throws excetption if it cannot connect 
	 *	to the host; for example, invalid hostname or 
	 *	no service running.
	 */
	public void connect(String server, int port, String login) throws IOException{
		setName(login);	connect(server, port);	
	}

	/**
	 * Disconnects if the WTalkProtocol is connected.
	 */
	public void disconnect(String msg){
		mSession=null;
		popc = new POP3Checker();
	}

	/**
	 * Returns whether the WTalkProtocol object has an active connection. 
	 * 
	 * @return Returns true if the WTalkProtocol has an active connection.
	 */
	public boolean isConnected(){ return (mSession!=null);}	// Are we currently connected?


	/**
	 * Sets the user's return address.  Anythning before an initial 
	 * '@' sign is treated as a login.
	 *
	 * @param name User's return address.
	 */
	public void setName(String name) { 
		StringTokenizer st = new StringTokenizer(name, "@");
		mName=name;
		mLogin=st.nextToken();
		mHost=name.substring(mLogin.length()+1);
		popc.setLogin(mLogin);
		if(mHost!=null && !mHost.equals("")) popc.setServer(mHost);

System.out.println("Name set to: " + name + "\tLogin: " + mLogin + "\tServer: " + mHost);
	}

	/**
	 * Returns the user's name (handle) 
	 *	understood by the current connection.
	 *
	 * @return The user's current handle on this connection.
	 */
	public String getName(){ return mName; }

	/**
	 * Sets the password the connection will use to authenticate.
	 * <p>
	 * Note: this is not the same as 
	 *	<a href="#changePassword"><code>changePassword</code></a>.
	 *
	 * @param passwd Password to use to authenticate.
	 */
	public void setPassword(String passwd) { 
		mPasswd = passwd; 
		popc.setPasswd(passwd);
	}

	/**
	 * Not supported.
	 *
	 * @param oldpwd The old password, such as the one used to authenticate
	 *	with when loggin on.
	 * @param newpwd A new password, to be used in future authentication
	 * 	attempts.
	 */
	public void changePassword(String oldpwd, String newpwd){} 

	/**
	 * Sets the server to use when <code>connect()</code> is called.
	 * <p>
	 * This function <strong>SHOULD NOT</strong> attempt to connect.
	 *
	 * @param server Host that <code>connect()</code> will connect to.
	 */
	public void setServer(String server) {	
		// This only works if POP and SMTP servers are the same...
		mServer = server;

		// This might fix above comment .... 
		// hopefully it won't break things
		if(mHost!=null && !mHost.equals("")) popc.setServer(mHost);
		else popc.setServer(server);
	}

	/**
	 * Returns the name of the current server that <code>connect()</code>
	 * 	will use.  This is often the server we are connected to
	 *	but may not be if <code>setServer()</code> has been called
	 *	since we have connected.
	 *
	 * @return Name of current server WTalkProtocol has stored.
	 */
	public String getServer(){ 	// returns name of current server
		return mServer;
	}

	/**
	 * Sets the port to use when connecting to a host.
	 * <p>At the moment,  this doesn't do anything because
	 * I am unsure how to handle it (considering POP and SMTP are on
	 * different ports...
	 *
	 * @param port Port number to use when connecting. 
	 * 	<p>
	 *	If port==0, then the protocol should use a "Default" port.
	 */
	public void setServerPort(int port){}

	/**
	 * Returns current Port; the last port set by setServerPort().
	 * Note that this may return 0, indicating the default port
	 * is being used.
	 *
	 * @return The current Port stored by the WTalkProtocol.
	 */
	public int getServerPort(){ return 0; }
	//
	//////////////////////////////////////////////////////////


	//////////////////////////////////////////////////////////
	//
	// Communication functions
	//

	//
	// IM functions
	//
	/**
	 * Sends a "normal" text message, data.
	 *
	 * @param to Target of message.
	 * @param data Message contents.
	 */
	public void sendMessage(String to, String data) {
		try{
			MimeMessage message = new MimeMessage(mSession);
			message.setText(data);
			message.setSubject("");
			message.setFrom(new InternetAddress(mName));
			message.addRecipient(Message.RecipientType.TO, 
					new InternetAddress(to));

			// Just some headers ...
			message.setHeader("X-Mailer", wtalk.WTalk.getBanner());
			message.setHeader("X-PoweredBy", getBanner());
			message.setHeader("X-URL", URL);

			message.setSentDate(new Date());
			Transport.send(message);


			if(sph!=null) sph.handle(to, null, data);
			else if (dmh!=null) dmh.handle(to, null, data);
		}catch(javax.mail.MessagingException me){
			// print: message couldn't be sent
			RMsgHandler r = null;
			r=smh!=null?smh:dmh;
			if(r!=null)r.handle(to,null,"Error Sending Message==Message was not sent.");
		}
	}

	/**
	 * Requests information on "who is &lt;<em>user</em>&gt;".
	 *
	 * @param user Name of user to requst information on.
	 */
	public void whois(String user){}

	/**
	 * Adds a buddy to a "Buddy List" stored or managed by
	 *	either ther server or WTalk, as opposed to 
	 *	the shell.
	 *
	 * @param buddy Buddy's name to be added.
	 * @param group Group to which Buddy will be added.
	 */
	public void addBuddy(String buddy, String group){}

	/**
	 * Removes a buddy from a "Buddy List" stored or managed by
	 *	either the server or WTalk, as opposed to
	 *	the shell.
	 *
	 * @param buddy Buddy's which will be removed.
	 * @param group Group from which Buddy will be removed.
	 */
	public void removeBuddy(String buddy, String group){}
	//listBuddies();


	/**
	 * Sets away status and messages, or removes away status.
	 *
	 * @param message Sets <em>message</em> as away message. 
	 *	if <em>message</em> is null, away status is removed.
	 */
	public void setAway(String message){}

	//addDeny();
	//removeDeny();


	//
	// Chat functions
	//
	/**
	 * Joins chat <em>room</em>.
	 * <p>Not supported.
	 *
	 * @param room Name of chat to join.
	 */
	public void joinChat(String room){}

	/**
	 * Leaves (parts) chat <em>room</em>.
	 * <p>Not supported.
	 *
	 * @param room Name of chat to part.
	 */
	public void partChat(String room){} 


	/**
	 * Sends a message to all members in <em>room</em>.
	 * <p>Not supported.
	 *
	 * @param room The room to which a message is sent.
	 * @param data The contents of the message.
	 */
	public void chatMessage(String room, String data){}

	/**
	 * Sets or retrieves the topic of <em>room</em>.
	 * <p>Not supported.
	 *
	 * @param room Room whose topic we are concerned.
	 * @param topic If not null, the topic which will be
	 *	set in <em>room</em>.  If null, a request
	 * 	to receive the current topic is sent.
	 */
	public void setTopic(String room, String topic){} 

	/**
	 * Requests information on a chat, <em>room</em>, or all
	 * 	chats on the server [system].
	 * <p>Not supported.
	 *
	 * @param room The chat which we would like information.
	 *	A null value indicates a request for all active chats
	 *	if this is permitted by the protocol (it may not be).
	 */
	public void chatList(String room){}	// room may be null!


	//
	// Misc. messaging
	//
	/**
	 * Sends a brodcast (wall) message to all users.
	 * <p>Not supported.
	 *
	 * @param msg The message to brodcast.
	 */
	public void sendWall(String msg){}
	public void sendSub(String to, String data){};
	//
	//////////////////////////////////////////////////////////



	//////////////////////////////////////////////////////////
	//
	// Admin functions
	//
	/**
	 * Sets or removes a ban on <em>user</em>.
	 * <p>Not supported.
	 *
	 * @param user The user with whose bannedness we are concerned.
	 * @param banned A true value bans <em>user</em>, false unbans <em>uer</em>.
	 */
	public void setBan(String user, boolean banned){}
	//
	//////////////////////////////////////////////////////////
}
